@@ -1,6 +1,6 @@
 /*
- *  methods for LUKS-related key-management for cryptmount
- *  (C)Copyright 2005-2010, RW Penney
+ *  Methods for LUKS-related key-management for cryptmount
+ *  (C)Copyright 2005-2011, RW Penney
  */
 
 /*
@@ -58,8 +58,8 @@ typedef struct {
 #    include "luks/internal.h"
 #    include "luks/luks.h"
 #    if HAVE_UUID
-#      include <uuid/uuid.h>
-#  endif
+#      include <uuid.h>
+#    endif
 
 extern struct setup_backend setup_libdevmapper_backend;
 
@@ -122,8 +122,11 @@ static void kmluks_splitmode(const char *fullname, char **cipher, char **mode)
 
 static int kmluks_init_algs()
 {   static int done_secmem = 0;
+    int flg = 0;
     const char *GCRYPT_REQ_VERSION="1.1.42";
 
+    flg = init_crypto(NULL);
+
     if (!done_secmem || !gcry_control(GCRYCTL_INITIALIZATION_FINISHED_P)) {
         if (!gcry_check_version(GCRYPT_REQ_VERSION)) return -1;
 
@@ -132,14 +135,14 @@ static int kmluks_init_algs()
          * gcrypt secmem facilities will also drop setuid privileges,
          * which would conflict with device-mapper system calls
          * within cryptmount */
-		(void)gcry_control(GCRYCTL_DISABLE_SECMEM);
+        (void)gcry_control(GCRYCTL_DISABLE_SECMEM);
 
         (void)gcry_control(GCRYCTL_INITIALIZATION_FINISHED, 0);
 
         done_secmem = 1;
     }
 
-    return 0;
+    return flg;
 }
 
 
@@ -207,7 +210,7 @@ static int kmluks_get_key(bound_tgtdefn_t *boundtgt,
     const char *tgtdev = NULL;
     struct crypt_device *luks_ctxt = NULL;
     struct luks_phdr *lukshdr;
-    struct luks_masterkey *lukskey = NULL;
+    struct volume_key *lukskey = NULL;
     int killloop = 0, slot, eflag = ERR_NOERROR;
     luks_overrides_t *luksor;
     int64_t delta;
@@ -251,8 +254,8 @@ static int kmluks_get_key(bound_tgtdefn_t *boundtgt,
     boundtgt->km_data = (void*)luksor;
 
     /* Take copy of LUKS master-key: */
-    *keylen = lukskey->keyLength;
-    *key = (unsigned char*)sec_realloc((void*)*key, (size_t)lukskey->keyLength);
+    *keylen = lukskey->keylength;
+    *key = (unsigned char*)sec_realloc((void*)*key, (size_t)lukskey->keylength);
     memcpy((void*)*key, (const void*)lukskey->key, (size_t)*keylen);
 
   bail_out:
@@ -260,7 +263,7 @@ static int kmluks_get_key(bound_tgtdefn_t *boundtgt,
     unblockify_file(&tgtdev, killloop);
     udev_settle();
     if (passwd != NULL) sec_free((void*)passwd);
-    if (lukskey != NULL) LUKS_dealloc_masterkey(lukskey);
+    if (lukskey != NULL) crypt_free_volume_key(lukskey);
     if (luks_ctxt != NULL) crypt_free(luks_ctxt);
 
     return eflag;
@@ -278,7 +281,7 @@ static int kmluks_put_key(bound_tgtdefn_t *boundtgt,
     const char *tgtdev = NULL;
     struct crypt_device *luks_ctxt = NULL;
     struct luks_phdr *lukshdr;
-    struct luks_masterkey *lukskey = NULL;
+    struct volume_key *lukskey = NULL;
     unsigned keyslot = 0;
     uint64_t *iteration_time = NULL, *PBKDF2_per_sec = NULL;
     luks_overrides_t *luksor = NULL;
@@ -313,16 +316,19 @@ static int kmluks_put_key(bound_tgtdefn_t *boundtgt,
     crypt_init(&luks_ctxt, tgtdev);
     lukshdr = crypt_get_lukshdr(luks_ctxt);
     crypt_get_pbkdf2params(luks_ctxt, &iteration_time, &PBKDF2_per_sec);
-    lukskey = LUKS_alloc_masterkey(keylen, (const char*)key);
+    lukskey = crypt_alloc_volume_key(keylen, (const char*)key);
 
     if (formatting) {
         /* Format new partition */
-        keyslot = 0;
+        unsigned long reqd_align = DEFAULT_DISK_ALIGNMENT, align_offs = 0;
 
+        keyslot = 0;
+        get_topology_alignment(tgtdev, &reqd_align,
+                                &align_offs, DEFAULT_DISK_ALIGNMENT);
         kmluks_splitmode(boundtgt->tgt->cipher, &ciphername, &ciphermode);
         r = LUKS_generate_phdr(lukshdr, lukskey, ciphername, ciphermode,
-                            boundtgt->tgt->key.digestalg, NULL,
-                            LUKS_STRIPES, DEFAULT_ALIGNMENT,
+                            boundtgt->tgt->key.digestalg, NULL, LUKS_STRIPES,
+                            reqd_align / SECTOR_SIZE, align_offs / SECTOR_SIZE,
                             *iteration_time, PBKDF2_per_sec, luks_ctxt);
         if (r < 0) {
             fprintf(stderr, _("Failed to create LUKS header for \"%s\"\n"),
@@ -358,7 +364,7 @@ static int kmluks_put_key(bound_tgtdefn_t *boundtgt,
 
     unblockify_file(&tgtdev, killloop);
     udev_settle();
-    if (lukskey != NULL) LUKS_dealloc_masterkey(lukskey);
+    if (lukskey != NULL) crypt_free_volume_key(lukskey);
     if (passwd != NULL) sec_free((void*)passwd);
     if (ciphername != NULL) free((void*)ciphername);
     if (ciphermode != NULL) free((void*)ciphermode);
@@ -460,5 +466,5 @@ keymanager_t *kmluks_gethandle()
 #endif  /* !AS_MODULE */
 
 /*
- *  (C)Copyright 2005-2010, RW Penney
+ *  (C)Copyright 2005-2011, RW Penney
  */
