@@ -1,6 +1,6 @@
 /*
  *  Methods for encryption/security mechanisms for cryptmount
- *  (C)Copyright 2005-2010, RW Penney
+ *  (C)Copyright 2005-2011, RW Penney
  */
 
 /*
@@ -430,7 +430,10 @@ int km_test_managers(void)
     build_keymgrs();
 
     for (km=keymgrs; km!=NULL; km=km->next) {
-        if (km->run_tests != NULL) flg |= km->run_tests();
+        if (km->run_tests == NULL) continue;
+
+        km->init_algs();
+        flg |= km->run_tests();
     }
 
 #if 0
@@ -842,5 +845,5 @@ int cm_mutex_unlock(void)
 }
 
 /*
- *  (C)Copyright 2005-2010, RW Penney
+ *  (C)Copyright 2005-2011, RW Penney
  */
