@@ -1,6 +1,6 @@
 /*
  *  Filesystem-related utilities for cryptmount
- *  (C)Copyright 2005-2010, RW Penney
+ *  (C)Copyright 2005-2011, RW Penney
  */
 
 /*
@@ -25,6 +25,7 @@
 
 #include <ctype.h>
 #include <fcntl.h>
+#include <math.h>
 #include <mntent.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -48,8 +49,8 @@
 #endif
 
 
-#define ETCMTAB     "/etc/mtab"
-#define ETCMTABTMP  "/etc/mtab.cm"
+static const char *ETCMTAB = "/etc/mtab",
+                  *ETCMTABTMP = "/etc/mtab.cm";
 
 
 enum {
@@ -61,7 +62,7 @@ int do_rmvmntent(const tgtdefn_t *tgt);
 
 
 static char **split_fsckopts(const char *fsckoptions, unsigned *argc)
-    /* Split options separated by ';' into vector of strings */
+    /** Split options separated by ';' into vector of strings */
 {   const char SEP = ';';
     const char *pos, *posnext;
     size_t optlen;
@@ -101,7 +102,7 @@ static char **split_fsckopts(const char *fsckoptions, unsigned *argc)
 #if ERSATZ_MOUNT
 
 static int parse_mountoptions(const char *buff, unsigned long *mflags)
-    /* Convert string of mount-options into binary flags */
+    /** Convert string of mount-options into binary flags */
 {   struct fsopt_t {
         const char *str;
         unsigned long mask; };
@@ -141,7 +142,7 @@ static int parse_mountoptions(const char *buff, unsigned long *mflags)
 
 
 int fs_mount(const char *mntdev, const tgtdefn_t *tgt)
-    /* Fallback version of mount(8) using mount(2) */
+    /** Fallback version of mount(8) using mount(2) */
 {   unsigned long mflags;
     int eflag=ERR_NOERROR;
     char *errstr=NULL;
@@ -168,7 +169,7 @@ int fs_mount(const char *mntdev, const tgtdefn_t *tgt)
 #else   /* !ERSATZ_MOUNT */
 
 int fs_mount(const char *dev, const tgtdefn_t *tgt)
-    /* Delegate filesystem mounting to mount(8) */
+    /** Delegate filesystem mounting to mount(8) */
 {   int idx, stat, eflag=ERR_NOERROR;
     const char *argv[16];
 
@@ -196,7 +197,7 @@ int fs_mount(const char *dev, const tgtdefn_t *tgt)
 #if ERSATZ_UMOUNT
 
 int fs_unmount(const tgtdefn_t *tgt)
-    /* Fallback version of umount(8) using umount(2) */
+    /** Fallback version of umount(8) using umount(2) */
 {   int eflag=ERR_NOERROR;
     char *errstr=NULL;
 
@@ -218,7 +219,7 @@ int fs_unmount(const tgtdefn_t *tgt)
 #else   /* !ERSATZ_UMOUNT */
 
 int fs_unmount(const tgtdefn_t *tgt)
-    /* Delegate filesystem mounting to umount(8) */
+    /** Delegate filesystem mounting to umount(8) */
 {   int idx, stat, eflag=ERR_NOERROR;
     const char *argv[16];
 
@@ -248,9 +249,12 @@ int fs_unmount(const tgtdefn_t *tgt)
 #if WITH_CSWAP
 
 int fs_swapon(const char *mntdev, const tgtdefn_t *tgt)
-{   int idx, stat, eflag=ERR_NOERROR;
+    /** Install \a mntdev as a new swap device */
+{   int idx, stat, prio, rawprio = 0x100, expendable = 0, eflag=ERR_NOERROR;
     const char *argv[8];
-    int prio, rawprio=0x100;
+    double cry_entropy = 0.0, raw_entropy = 0.0;
+    const size_t entrosize = 1 << 20;
+    const double blank_thresh = 1e-5, noise_thresh = 7.0;
 
     if (strcmp(tgt->fstype,"swap") != 0) {
         fprintf(stderr, _("Unsuitable filesystem type \"%s\" for swapping\n"),
@@ -259,16 +263,31 @@ int fs_swapon(const char *mntdev, const tgtdefn_t *tgt)
         goto bail_out;
     }
 
-    if ((tgt->flags & FLG_MKSWAP) != 0) {
-        /* construct argument list for mkswap <dev> */
-        idx = 0;
-        argv[idx++] = "[cryptmount-mkswap]";
-        argv[idx++] = mntdev;
-        argv[idx++] = NULL;
+    /*  Measure entropy of filesystem and underlying raw device
+        to check whether it's safe to force mkswap to format
+        what it thinks is a whole disk(!), or which could be
+        a file containing wanted data: */
+    raw_entropy = fs_entropy(tgt->dev, entrosize);
+    cry_entropy = fs_entropy(mntdev, entrosize);
+    expendable = (raw_entropy < blank_thresh)
+                || (cry_entropy < blank_thresh)
+                || (raw_entropy > noise_thresh && cry_entropy > noise_thresh);
 
-        stat = run_sucommand(DLGT_MKSWAP, argv, F_CLOSE1);
-        eflag = (stat != 0 ? ERR_BADSWAP : ERR_NOERROR);
-        if (eflag != ERR_NOERROR) goto bail_out;
+    if ((tgt->flags & FLG_MKSWAP) != 0) {
+        if (expendable) {
+            /* Construct argument list for mkswap <dev> */
+            idx = 0;
+            argv[idx++] = "[cryptmount-mkswap]";
+            if (expendable) argv[idx++] = "-f";
+            argv[idx++] = mntdev;
+            argv[idx++] = NULL;
+
+            stat = run_sucommand(DLGT_MKSWAP, argv, F_CLOSE1);
+            eflag = (stat != 0 ? ERR_BADSWAP : ERR_NOERROR);
+            if (eflag != ERR_NOERROR) goto bail_out;
+        } else {
+            fprintf(stderr, _("Device \"%s\" appears to contain data (entropy=%.3g,%.3g) - please run mkswap manually\n"), tgt->dev, raw_entropy, cry_entropy);
+        }
     }
 
     if (tgt->mountoptions != NULL
@@ -316,7 +335,7 @@ int fs_swapoff(const tgtdefn_t *tgt)
 
 
 int do_addmntent(const tgtdefn_t *tgt)
-    /* Add entry into /etc/mtab for newly-mounted filing system */
+    /** Add entry into /etc/mtab for newly-mounted filing system */
 {   char *mntdev=NULL;
     struct mntent mntinfo;
     FILE *fp;
@@ -348,7 +367,7 @@ int do_addmntent(const tgtdefn_t *tgt)
 
 
 int do_rmvmntent(const tgtdefn_t *tgt)
-    /* Remove entry from /etc/mtab after unmounting filing system */
+    /** Remove entry from /etc/mtab after unmounting filing system */
 {   char *mntdev=NULL;
     struct mntent *mntinfo;
     FILE *fp_in=NULL,*fp_out=NULL;
@@ -410,7 +429,7 @@ int do_rmvmntent(const tgtdefn_t *tgt)
 
 
 int is_mounted(const tgtdefn_t *tgt)
-    /* Check if target is mounted */
+    /** Check if target is mounted */
 {   int mounted=0;
     char *mntdev=NULL;
     struct mntent *mntinfo;
@@ -445,7 +464,7 @@ int is_mounted(const tgtdefn_t *tgt)
 
 
 int is_readonlyfs(const char *path)
-    /* check if filesystem containing *path is read-only */
+    /** Check if filesystem containing *path is read-only */
 {   struct statvfs sbuff;
 
     return (path == NULL
@@ -457,7 +476,7 @@ int is_readonlyfs(const char *path)
 #if WITH_FSCK
 
 int fs_check(const char *dev, const tgtdefn_t *tgt)
-    /* Run 'fsck' on target filesystem */
+    /** Run 'fsck' on target filesystem */
 {   int idx, stat, eflag=ERR_NOERROR;
     unsigned pos, n_opts = 0;
     const char **opts = NULL, **argv = NULL;
@@ -493,11 +512,53 @@ int fs_check(const char *dev, const tgtdefn_t *tgt)
 #endif  /* WITH_FSCK */
 
 
+double fs_entropy(const char *dev, const size_t blklen)
+    /** Calculate degree of randomness on initial block of filesystem */
+{   unsigned *counts = NULL;
+    unsigned char buff[4096];
+    size_t chunk, totcount = 0;
+    int fd, step, i;
+    double plogp = 0.0;
+
+    fd = open(dev, O_RDONLY | O_NOATIME);
+    if (fd < 0) return -1.0;
+
+    counts = (unsigned*)calloc((size_t)256, sizeof(counts[0]));
+
+    while (totcount < blklen) {
+        chunk = blklen - totcount;
+        if (chunk > sizeof(buff)) chunk = sizeof(buff);
+
+        step = read(fd, (void*)buff, chunk);
+        if (step <= 0) break;
+
+        for (i=0; i<step; ++i) ++counts[buff[i]];
+
+        totcount += step;
+    }
+
+    if (totcount > 0) {
+        for (i=0; i<256; ++i) {
+            if (counts[i] == 0) continue;
+            plogp -= counts[i] * log((double)counts[i]);
+        }
+        plogp = (plogp / totcount) + log((double)totcount);
+    }
+
+    plogp /= log(2.0);
+
+    free((void*)counts);
+    close(fd);
+
+    return plogp;
+}
+
+
 #if !ERSATZ_MOUNT || !ERSATZ_UMOUNT || WITH_FSCK || WITH_CSWAP
 
 static int run_sucommand(const char *path, const char **argv,
                         unsigned switches)
-    /* fork (& wait for) system-command as root */
+    /** fork (& wait for) system-command as root */
 {   pid_t child;
     int stat=-1, fd;
 
@@ -579,10 +640,47 @@ int fs_test_splitopts()
     CM_TEST_OK();
 }
 
+
+/**
+ *  Check calculation of entropy of files containing
+ *  uniformly distributed sequences of bytes.
+ */
+int fs_test_entropy()
+{   char fname[256];
+    unsigned i, nbits, mask, offset;
+    size_t count;
+    FILE *fp;
+    double S;
+    int its = 1000;
+
+    CM_TEST_START("Entropy calculation");
+
+    sprintf(fname, "/tmp/cm-%u-entropy", (unsigned)getpid());
+
+    while (its--) {
+        nbits = (rand() & 7) + 1;
+        mask = (1 << nbits) - 1;
+
+        count = (1 << nbits) * ((rand() & 31) + 1);
+        offset = (rand() & 0xff);
+
+        fp = fopen(fname, "wb");
+        for (i=0; i<count; ++i) {
+            fputc((i + offset) & mask, fp);
+        }
+        fclose(fp);
+
+        S = fs_entropy(fname, count);
+        if (fabs(nbits - S) > 1e-2) CM_TEST_FAIL("Mismatch");
+    }
+
+    (void)unlink(fname);
+
+    CM_TEST_OK();
+}
+
 #endif  /* TESTING */
 
-/* vim: set ts=4 sw=4 et: */
-
 /*
- *  (C)Copyright 2005-2010, RW Penney
+ *  (C)Copyright 2005-2011, RW Penney
  */
